package generated.A0000000P;

import java.net.Socket;
import java.io.*;

/**
 * Streaming stub for org.objectweb.dsrg.connector.streaming.StreamProvider.
 * (real target iface is org.objectweb.dsrg.connector.streaming.StreamProvider, implementing interface is org.objectweb.dsrg.connector.streaming.StreamProvider)
 * 
 * A stub element has a client remote port "line" through which
 * it connects to the target object. When binding the "line" port
 * to a remote target using a bundle of references, a local stub
 * is only interested in the "local" reference.
 *
 * TODO: call reconfiguration handler when the target references change.
 */
public final class StreamingStub 
    implements
        org.objectweb.dsrg.connector.ElementLocalServer,
        org.objectweb.dsrg.connector.ElementRemoteClient,
        org.objectweb.dsrg.connector.ReconfigurableElement,
        org.objectweb.dsrg.connector.streaming.StreamProvider {

	protected String remoteHost = null;
	protected int remotePort = -1;
	
    /**
     * A bundle of remote references to the target of business method
     * invocations.
     */
    protected org.objectweb.dsrg.connector.RemoteRefBundle targetRemoteRef;

    /*
     * Instance of dock connector manager which is responsible for connector
     * units within the dock.
     */
    protected org.objectweb.dsrg.connector.mgr.DockConnectorManager dcm;

    /*
     * Reference to parent connector unit.
     */
    protected org.objectweb.dsrg.connector.ConnectorUnit parentUnit;

    protected boolean isTopLevel;

    /**
     * Reconfiguration request handler.
     */
    protected org.objectweb.dsrg.connector.ReconfigurationHandler reconfigurationHandler;


    /* **********************************************************************
     * CONSTRUCTORS
     * **********************************************************************/

    /**
     * Constructs a new local stub element.
     */
    public StreamingStub (
            org.objectweb.dsrg.connector.ConnectorUnit parentUnit,
            boolean isTopLevel) {

        this.parentUnit = parentUnit;
        this.isTopLevel = isTopLevel;

        dcm = org.objectweb.dsrg.connector.mgr.
            DockConnectorManagerHelper.getDockConnectorManager();
    }


    /* **********************************************************************
     * Element Methods
     * **********************************************************************/

	public final String getElementInfo (String indent) {
		StringBuilder result = new StringBuilder ();

		result.append (indent +"Implementation: \"streaming_stub\"\n");

		return result.toString ();
	}


    /**
     * Sets the reconfiguration handler.
     */
    public final void setElReconfigurationHandler (
        org.objectweb.dsrg.connector.ReconfigurationHandler reconfigurationHandler) {

        this.reconfigurationHandler = reconfigurationHandler;
    }


    /* **********************************************************************
     * ElementLocalServer Methods
     * **********************************************************************/

    /**
     * Queries given provided port for a local object reference.
     */
    public Object lookupElPort (String portName)
        throws org.objectweb.dsrg.connector.ElementLinkException {

        if ("call".equals (portName)) {
            Object portReference;
            
			/* handle all call by this element */
            portReference = this;

            if (isTopLevel) {
                dcm.reregisterConnectorUnitReference (
                    parentUnit, "call", portReference);
            }

            return portReference;

        } else {
            throw new org.objectweb.dsrg.connector.ElementLinkException (
                "Attempt to look up non-existent provided port '"+ portName +"'.");
        }
    }


    /* **********************************************************************
     * ElementRemoteClient Methods
     * **********************************************************************/

    /**
     * Binds a remote client port of this element to a remote server port
     * using a bundle of named references. This local stub has one remote
     * client port "line" and requires the reference bundle to contain a
     * reference named "local" which can be then resolved to a true local
     * object reference.
     *
     * TODO: Handle reconfiguration.
     */
    public void bindElRemotePort (
            String portName,
            org.objectweb.dsrg.connector.RemoteRefBundle refBundle)
        throws org.objectweb.dsrg.connector.ElementLinkException {

        if ("line".equals (portName)) {
            org.objectweb.dsrg.connector.RemoteRef streamingRef;

            streamingRef = refBundle.getRef ("streaming");
            if (streamingRef != null) {
                targetRemoteRef = refBundle;
                String[] refS = streamingRef.stringifiedRef.split(":");
                remoteHost = refS[0];
                remotePort = Integer.valueOf(refS[1]).intValue();

            } else {
                resetRemoteRef();
            }

        } else {
            throw new org.objectweb.dsrg.connector.ElementLinkException (
                "Attempt to bind non-existent remote port '"+ portName +"'.");
        }
    }


    /**
     * Unbinds a given remote client port of this element
     * from a remote server port.
     *
     * TODO: Handle reconfiguration.
     */
    public void unbindElRemotePort (String portName)
        throws org.objectweb.dsrg.connector.ElementLinkException {

        if ("line".equals (portName)) {            
            resetRemoteRef();            
        } else {
            throw new org.objectweb.dsrg.connector.ElementLinkException (
                "Attempt to unbind non-existent remote port '"+ portName +"'.");
        }
    }
    
    protected void resetRemoteRef() {
    	targetRemoteRef = null;
    	remoteHost = null;
        remotePort = -1;
    }


    /**
     * Queries given remote client port of this element
     * for its target reference bundle.
     */
    public org.objectweb.dsrg.connector.RemoteRefBundle getElRemoteTarget (
            String portName)
        throws org.objectweb.dsrg.connector.ElementLinkException {

        if ("line".equals (portName)) {
            return targetRemoteRef;

        } else {
            throw new org.objectweb.dsrg.connector.ElementLinkException (
                "Attempt to query target of non-existent remote port '"+ portName +"'.");
        }
    }


    /* **********************************************************************
     * org.objectweb.dsrg.connector.streaming.StreamProvider Methods
     * **********************************************************************/
	
	@Override
	public InputStream getStream(String id) {
		InputStream is = null;
		try {
			System.err.println("[client " + this +"] connecting to server " + targetRemoteRef);
			Socket socket = new Socket(remoteHost, remotePort);
			System.err.println("[client " + this +"] connection established: " + socket.isConnected());
			
			
			// send identification of stream
			ObjectOutputStream oos = new ObjectOutputStream(socket.getOutputStream());
			oos.writeObject(id);
			
			// received stream		
			is = socket.getInputStream(); // NOTE: closing the returned stream will close associated socket 
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return is;
	}	
	
//	class ProxyInputStream extends InputStream {
//		private InputStream remoteIs;
//		private Sock
//		public ProxyInputStream
//		
//	}

}
