package generated.A0000000R;

import java.net.ServerSocket;
import java.net.Socket;
import java.io.*;

/**
 * Streaming skeleton for org.objectweb.dsrg.connector.streaming.StreamProvider
 *
 * A skeleton element has a server remote port "line" through which
 * remote clients connect to the target object, which is bound to
 * the "call" required port.
 *
 */
public final class StreamingSkeleton
    implements
        org.objectweb.dsrg.connector.ElementRemoteServer,
        org.objectweb.dsrg.connector.ElementLocalClient,
        org.objectweb.dsrg.connector.ReconfigurableElement, 
        Runnable {

    /**
     * Local target of business method invocations.
     */
    protected org.objectweb.dsrg.connector.streaming.StreamProvider target;

    /**
     * Instance of dock connector manager which is responsible for connector
     * units within the dock.
     */
    protected final org.objectweb.dsrg.connector.mgr.DockConnectorManager dcm;

    /**
     * Reference to parent connector unit.
     */
    protected final org.objectweb.dsrg.connector.ConnectorUnit parentUnit;

    protected boolean isTopLevel;

    /**
     * Reconfiguration request handler.
     */
    protected org.objectweb.dsrg.connector.ReconfigurationHandler reconfigurationHandler;
    
    /**
     * Socket for remote connections.
     */
    protected ServerSocket serverSocket = null;
    protected Thread serverThread = null;
    /* FIXME how to capture life cycle of the connector ? */
    protected boolean quitServerThread = false;

    public static final int BUFFER_SIZE = 4096;

    /* **********************************************************************
     * CONSTRUCTORS
     * **********************************************************************/

    /**
     * Creates a new local skeleton element.
     */
    public StreamingSkeleton (
            org.objectweb.dsrg.connector.ConnectorUnit parentUnit,
            boolean isTopLevel) {

        this.parentUnit = parentUnit;
        this.isTopLevel = isTopLevel;

        dcm = org.objectweb.dsrg.connector.mgr.
            DockConnectorManagerHelper.getDockConnectorManager ();
        
        // initialize socket
        try {
        	init();
        } catch (Exception e) {
        	throw new org.objectweb.dsrg.connector.ConnectorTransportException("Cannot initialize socket", e);
        }
    }
    
    protected void init() throws IOException {
    	serverSocket = new ServerSocket(0);
    	System.err.println("[server " + this + "] socket created at " + serverSocket.getInetAddress().getHostAddress() + ":" + serverSocket.getLocalPort());
    	
    	serverThread = new Thread(this);
    	serverThread.start();
    }


    /* **********************************************************************
     * Element Methods
     * **********************************************************************/

	public final String getElementInfo (String indent) {
		StringBuilder result = new StringBuilder ();

		result.append (indent +"Implementation: \"streaming_skeleton\"\n");

		return result.toString ();
	}


    /**
     * Sets the reconfiguration handler.
     */
    public final void setElReconfigurationHandler (
        org.objectweb.dsrg.connector.ReconfigurationHandler reconfigurationHandler) {

        this.reconfigurationHandler = reconfigurationHandler;
    }


    /* **********************************************************************
     * ElementLocalClient Methods
     * **********************************************************************/

    /**
     * Binds a given required port of this element to a target object using
     * a local object reference.
     *
     * TODO: Handle reconfiguration.
     */
    public void bindElPort (String portName, Object target)
        throws org.objectweb.dsrg.connector.ElementLinkException {

        if ("call".equals (portName)) {
            this.target = (org.objectweb.dsrg.connector.streaming.StreamProvider) target;

        } else {
            throw new org.objectweb.dsrg.connector.ElementLinkException (
                "Attempt to bind non-existent required port '"+ portName +"'.");
        }
    }


    /**
     * Unbinds a given required port of this element from its target.
     *
     * TODO: Handle reconfiguration.
     */
    public void unbindElPort (String portName)
        throws org.objectweb.dsrg.connector.ElementLinkException {

        if ("call".equals (portName)) {
            this.target = null;

        } else {
            throw new org.objectweb.dsrg.connector.ElementLinkException (
                "Attempt to unbind non-existent required port '"+ portName +"'.");
        }
    }


    /* **********************************************************************
     * ElementRemoteServer Methods
     * **********************************************************************/

    /**
     * Provides a bundle of remote references to the given remote server port.
     * Since this is a local skeleton, "local" reference is returned in the
     * reference bundle.
     */
    public org.objectweb.dsrg.connector.RemoteRefBundle lookupElRemotePort (
            String portName)
        throws org.objectweb.dsrg.connector.ElementLinkException {

        org.objectweb.dsrg.connector.RemoteRefBundle result =
            new org.objectweb.dsrg.connector.RemoteRefBundle ();

        if ("line".equals (portName)) {
            Object portReference;
           
            result.addRef (
                new org.objectweb.dsrg.connector.RemoteRef (
                    "streaming",
                    serverSocket.getInetAddress().getHostAddress() + ":" + serverSocket.getLocalPort()
                )
            );

        } else {
            throw new org.objectweb.dsrg.connector.ElementLinkException (
                "Attempt to look up non-existent remote port '"+ portName +"'.");
        }

        return result;
    }


    /**
     * Returns the names of remote ports supported by this element.
     */
    public String [] listElRemotePorts () {
        return new String [] { "line" };
    }
    
    @Override
	public void run() {
    	while(true) {
    		try {
    			Socket client = serverSocket.accept();
    			
    			/* transfer responsiblity of this socket to the handler method, block until client is served */
    			handleClientConnection(client);    			
    		
    		} catch (Exception e) {
    			System.err.println("[server "+this+"] some error occured:");
    			e.printStackTrace();
    			System.err.println("[server "+this+"] waiting for next client");
    		}
    		
    		if (quitServerThread) 
    			break;
    	}    	    	
    }
    
    protected void handleClientConnection(Socket client) throws Exception {
    	ObjectInputStream ois = new ObjectInputStream(client.getInputStream());
		String param = (String) ois.readObject();
		
		InputStream is = target.getStream(param);
		OutputStream os = client.getOutputStream();
				
		synchronized (is) {
			byte[] buffer = new byte[BUFFER_SIZE];
			int len= 0;
			while ((len = is.read(buffer, 0, BUFFER_SIZE)) != -1) {
				os.write(buffer,0, len);								
			}			
		}
		
		client.shutdownOutput();
		os.close();
		is.close();			    	
    }
}
