package generated.A0000001L;

/**
 * Composite connector element implementation. A composite element contains
 * instances of subelements and is responsible for bindings among those
 * elements. When building the architecture, ports of this composite element
 * are delegated/subsumed to ports of the subelements.
 */
public final class ServerUnit 
    implements
        org.objectweb.dsrg.connector.ElementLocalServer,
        org.objectweb.dsrg.connector.ElementLocalClient,
        org.objectweb.dsrg.connector.ElementRemoteServer,
        org.objectweb.dsrg.connector.ElementRemoteClient,
        org.objectweb.dsrg.connector.ReconfigurableElement,
        org.objectweb.dsrg.connector.ReconfigurationHandler {

    /**
     * The instances of subelements of this composite connector element.
     */
    protected org.objectweb.dsrg.connector.Element [] subElements;

    /**
     * Remote targets to which the remote ports are bound to.
     *
     * Even though the binding methods are delegated to subelements
     * which subsume their remote port to this element's remote port,
     * a query for a remote port target is best answered from this
     * element, so that we don't have to decide which of the subelements
     * to ask for the remote target reference bundle.
     */
    protected org.objectweb.dsrg.connector.RemoteRefBundle [] remoteTargetRefs;

    /*
     * Instance of dock connector manager which is responsible for connector
     * units within the dock.
     */
    protected final org.objectweb.dsrg.connector.mgr.DockConnectorManager dcm;

    /*
     * Reference to parent connector unit.
     */
    protected final org.objectweb.dsrg.connector.ConnectorUnit parentUnit;

    protected boolean isTopLevel;

    /**
     * Reconfiguration request handler.
     */
    protected org.objectweb.dsrg.connector.ReconfigurationHandler reconfigurationHandler;


    /* **********************************************************************
     * CONSTRUCTORS
     * **********************************************************************/

    /**
     * Creates a new composite element. Instantiation and
     * binding of subelements is decoupled from element creation.
     */
    public ServerUnit (
            org.objectweb.dsrg.connector.ConnectorUnit parentUnit,
            boolean isTopLevel)
        throws org.objectweb.dsrg.connector.ElementLinkException {

        this.parentUnit = parentUnit;
        this.isTopLevel = isTopLevel;

        dcm = org.objectweb.dsrg.connector.mgr.
            DockConnectorManagerHelper.getDockConnectorManager ();

        initializeArchitecture ();
    }


    /* **********************************************************************
     * Composite Methods
     * **********************************************************************/

	private void initializeArchitecture ()
		throws org.objectweb.dsrg.connector.ElementLinkException {

		subElements = new org.objectweb.dsrg.connector.Element [1];
		remoteTargetRefs = new org.objectweb.dsrg.connector.RemoteRefBundle [1];

		// instantiate subelements
		try {
			// inst skeleton, type skeleton, impl orig_local_skeleton
			subElements [0] = new generated.A0000001M.LocalSkeleton (parentUnit, false);

		} catch (Exception e) {
			throw new org.objectweb.dsrg.connector.ElementLinkException (e);
		}

		// distribute reconfiguration handler to server subelements
		distributeReconfigurationHandler ();
	}



    /**
     * Distributes a reference to this element as a reference to
     * reconfiguration handler to all reconfigurable subelements.
     */
    protected final void distributeReconfigurationHandler () {
        for (org.objectweb.dsrg.connector.Element elem : subElements) {
            if (elem instanceof org.objectweb.dsrg.connector.ReconfigurableElement) {
                ((org.objectweb.dsrg.connector.ReconfigurableElement) elem).
                    setElReconfigurationHandler (this);
            }
        }
    }


    /* **********************************************************************
     * Element Methods
     * **********************************************************************/

	public String getElementInfo (String indent) {
		StringBuilder result = new StringBuilder ();

		result.append (indent +"Implementation: \"orig_server_unit\"\n");
		result.append (indent +"    Sub-element: \"skeleton\"\n");
		result.append (subElements [0].getElementInfo (indent +"        "));

		return result.toString ();
	}



    /* **********************************************************************
     * ElementLocalServer Methods
     * **********************************************************************/

	public final Object lookupElPort (String portName)
		throws org.objectweb.dsrg.connector.ElementLinkException {

		{
			throw new org.objectweb.dsrg.connector.ElementLinkException (
				"Invalid provided port '"+ portName +"'.");
		}
	}



    /**
     * Sets the reconfiguration handler.
     */
    public final void setElReconfigurationHandler (
        org.objectweb.dsrg.connector.ReconfigurationHandler reconfigurationHandler) {

        this.reconfigurationHandler = reconfigurationHandler;
    }


    /* **********************************************************************
     * ElementLocalClient Methods
     * **********************************************************************/

	public final void bindElPort (String portName, Object target)
		throws org.objectweb.dsrg.connector.ElementLinkException {

		if ("call".equals (portName)) {
			// inst skeleton, type skeleton, impl orig_local_skeleton
			((org.objectweb.dsrg.connector.ElementLocalClient)
				subElements [0]).bindElPort ("call", target);

		} else {
			throw new org.objectweb.dsrg.connector.ElementLinkException (
				"Invalid required port '"+ portName +"'.");
		}
	}

	public final void unbindElPort (String portName)
		throws org.objectweb.dsrg.connector.ElementLinkException {

		if ("call".equals (portName)) {
			// inst skeleton, type skeleton, impl orig_local_skeleton
			((org.objectweb.dsrg.connector.ElementLocalClient)
				subElements [0]).unbindElPort ("call");

		} else {
			throw new org.objectweb.dsrg.connector.ElementLinkException (
				"Invalid required port '"+ portName +"'.");
		}
	}



    /* **********************************************************************
     * ElementRemoteServer Methods
     * **********************************************************************/

	public final org.objectweb.dsrg.connector.RemoteRefBundle lookupElRemotePort (
			String portName)
		throws org.objectweb.dsrg.connector.ElementLinkException {

		org.objectweb.dsrg.connector.RemoteRefBundle result =
			new org.objectweb.dsrg.connector.RemoteRefBundle ();

		if ("line".equals (portName)) {

			// inst skeleton, type skeleton, impl orig_local_skeleton
			if (subElements [0] instanceof org.objectweb.dsrg.connector.ElementRemoteServer) {
				// lookup skeleton.line
				result.addRefBundle (
					((org.objectweb.dsrg.connector.ElementRemoteServer)
					subElements [0]).lookupElRemotePort ("line"));
			}

		} else {
			throw new org.objectweb.dsrg.connector.ElementLinkException (
				"Invalid remote server port '"+ portName +"'.");
		}

		return result;
	}

	public final String [] listElRemotePorts () {
		return new String [] {
			"line"
		};
	}



    /* **********************************************************************
     * ElementRemoteClient Methods
     * **********************************************************************/

	public final void bindElRemotePort (
			String portName,
			org.objectweb.dsrg.connector.RemoteRefBundle refBundle)
		throws org.objectweb.dsrg.connector.ElementLinkException {

		if ("line".equals (portName)) {

			// inst skeleton, type skeleton, impl orig_local_skeleton
			if (subElements [0] instanceof org.objectweb.dsrg.connector.ElementRemoteClient) {
				// bind skeleton.line
				((org.objectweb.dsrg.connector.ElementRemoteClient)
					subElements [0]).bindElRemotePort ("line", refBundle);
			}

			remoteTargetRefs [0] = refBundle;

		} else {
			throw new org.objectweb.dsrg.connector.ElementLinkException (
				"Invalid remote client port '"+ portName +"'.");
		}
	}

	public final void unbindElRemotePort (String portName)
		throws org.objectweb.dsrg.connector.ElementLinkException {

		if ("line".equals (portName)) {

			// inst skeleton, type skeleton, impl orig_local_skeleton
			if (subElements [0] instanceof org.objectweb.dsrg.connector.ElementRemoteClient) {
				// unbind skeleton.line
				((org.objectweb.dsrg.connector.ElementRemoteClient)
					subElements [0]).unbindElRemotePort ("line");
			}

			remoteTargetRefs [0] = null;

		} else {
			throw new org.objectweb.dsrg.connector.ElementLinkException (
				"Invalid remote client port '"+ portName +"'.");
		}
	}

	public final org.objectweb.dsrg.connector.RemoteRefBundle getElRemoteTarget (
			String portName)
		throws org.objectweb.dsrg.connector.ElementLinkException {

		if ("line".equals (portName)) {
			return remoteTargetRefs [0];

		} else {
			throw new org.objectweb.dsrg.connector.ElementLinkException (
				"Invalid remote client port '"+ portName +"'.");
		}
	}



    /* **********************************************************************
     * ReconfigurationHandler Methods
     * **********************************************************************/

	public final void invalidateElPort (
			org.objectweb.dsrg.connector.ElementLocalServer element,
			String portName)
		throws org.objectweb.dsrg.connector.ReconfigurationException {

		try {

			{
				throw new org.objectweb.dsrg.connector.ElementLinkException (
					"Invalid reconfiguration request for port '"+ portName +"'.");
			}

		} catch (org.objectweb.dsrg.connector.ElementLinkException e) {
			throw new org.objectweb.dsrg.connector.ReconfigurationException (e);
		}
	}



    public final void invalidateElRemotePort (
            org.objectweb.dsrg.connector.ElementRemoteServer element,
            String portName)
        throws org.objectweb.dsrg.connector.ReconfigurationException {

        throw new org.objectweb.dsrg.connector.ReconfigurationException (
            "Reconfiguration not supported on remote ports, yet!");
    }

}
