package org.objectweb.dsrg.sofa.examples.logaspect;

import org.objectweb.dsrg.sofa.SOFAException;
import java.util.Map;
import org.objectweb.dsrg.sofa.microarchitecture.ComponentDescription;
import org.objectweb.dsrg.sofa.microarchitecture.SOFAMicroInterceptor;

/** Micro logger.
  *
  * <p><b>Note</b>: Uses SOFAMicroInterceptor insted of SOFAMicroComponent
  * for further use within control interface.</p>
  *
  * @author Ondrej Cerny
  */
public class MLogger implements MILog, SOFAMicroInterceptor {

	
	/** Writes log to console.
	  * 
	  */
	public void log(String component, MILog.Event e, MILog.Type t, MILog.View v, String method) {
		System.out.printf("LOG: COMPONENT: %s\n         EVENT: %s, %s, %s\n        METHOD: %s\n",
      component, e.toString(), t.toString(), v.toString(), method);
	}

  /* SOFAMicroComponent's methods: */

	/** Initializes this component.
	  * 
	  */
	public void init(ComponentDescription component, Map<String, String> parameters) throws SOFAException {
	}
	
	/** Returns itself as provided interface.
	  * 
	  */
	public Object getProvided(String name) throws SOFAException {
		if(!name.equals("log")) {
      throw new SOFAException("The microcomponent does not provide interface named "
        + ((name == null) ? "" : name));
		}
		return this;
	}

	/** Sets required interface (none).
	  * 
	  */
	public void setRequired(String name, Object ref) throws SOFAException {
		// we silently ignore interfaces we dont need
	}

  /* SOFAMicroInterceptor's methods: (for further use) */

	/** Returns itself as delegated interface.
	  *
	  */
	public Object getDelegatedProvided () throws SOFAException {
	    return this;
	}
	
	/** Sets delegated interface.
	  *
	  */
	public void setDelegatedRequired(Object ref) throws SOFAException {
	}
}
