/* $Id: StreamProducerImpl.java 278 2008-08-15 18:17:30Z mmalohlava $
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2008  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.objectweb.dsrg.sofa.examples.streaming.arch;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;

import org.objectweb.dsrg.connector.streaming.StreamProvider;
import org.objectweb.dsrg.sofa.examples.streaming.iface.MediaServer;

/**
 * Stream producer and also content information provider.
 * 
 * @author Michal Malohlava
 *
 */
public class StreamProducerImpl implements StreamProvider, MediaServer {
	/* name of media directory */
	public static final String MEDIA_DIR = "./media/";
	
	public static final String MEDIA_EXTENSIONS [] = { ".avi", ".mpg", ".mpeg", ".jpg" , ".jpeg", ".png", ".gif", ".mp3", ".wav"};

	@Override
	public InputStream getStream(String id) {
		InputStream is = null;
		File file = new File(MEDIA_DIR + id);
		
		try {
			is = new FileInputStream(file);
		} catch (FileNotFoundException e) {
			is = new EmptyInputStream();
		}		
		
		return is;
	}

	@Override
	public String getContentInfo(String id) {		
		return id;
	}

	@Override
	public String[] getContentList() {
		File dir = new File(MEDIA_DIR);
		
		if (!dir.isDirectory()) {
			System.err.println("[server compo] Directory " + MEDIA_DIR + " has to exist!");
			return new String[] {};
		} 
		
		return listContent(dir);
	}
	
	protected String[] listContent(File dir) {		
		String[] fileList = dir.list(new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				for (int i = 0; i < MEDIA_EXTENSIONS.length; i++) {
					if (name.endsWith(MEDIA_EXTENSIONS[i])) {						
						return true;
					}
				}
				return false;
			}			
		});
		
		return fileList;		
	}
	
	/**
	 * Dummy input stream for non-existing input source.
	 * 
	 * @author Michal Malohlava
	 *
	 */
	public class EmptyInputStream extends InputStream {
		
		@Override
		public int read() throws IOException {
			return -1;
		}
		
	}
	
}
