<?xml version="1.0" encoding="UTF-8"?>
<!-- $Id: simple2repository.xsl 52 2007-05-21 08:39:40Z hnetynka $ -->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	xmlns:org.objectweb.dsrg.sofa.repository.model="http:///org.objectweb.dsrg.sofa.repository.model">

	<xsl:strip-space elements="*" />
	<!-- <xsl:preserve-space elements=""/> -->
	<xsl:output method="xml" indent="yes" encoding="utf-8" />


	<!-- ================================================================================================================= -->
	<!-- Root elemenent                                                                                                    -->
	<!-- ================================================================================================================= -->
	<xsl:template match="/">
		<xsl:apply-templates />
	</xsl:template>


	<!-- ================================================================================================================= -->
	<!-- InterfaceType                                                                                                     -->
	<!-- ================================================================================================================= -->
	<xsl:template match="/itf-type">
		<org.objectweb.dsrg.sofa.repository.model:InterfaceType>
      <xsl:call-template name="handleInterfaceType" />
		</org.objectweb.dsrg.sofa.repository.model:InterfaceType>
	</xsl:template>
	<xsl:template match="/itf-type/dependency">
  </xsl:template>

  <xsl:template name="handleInterfaceType">
    <xsl:copy-of select="@*" />
  </xsl:template>


	<!-- ================================================================================================================= -->
	<!-- Frame                                                                                                             -->
	<!-- ================================================================================================================= -->
	<xsl:template match="/frame">
		<org.objectweb.dsrg.sofa.repository.model:Frame>
			<!--<xsl:attribute name="protocol">
				<xsl:value-of select="protocol/text()" />
				</xsl:attribute> -->
			<xsl:copy-of select="@name" />

			<xsl:if test="count(@top-level) = 1">
				<annotation xsi:type="org.objectweb.dsrg.sofa.repository.model:TopLevel" />
                        </xsl:if>

                        <xsl:if test="count(@service-tracker) = 1">
				<annotation xsi:type="org.objectweb.dsrg.sofa.repository.model:ServiceTracker">
                                    <xsl:attribute name="type">
                                        <xsl:value-of select="@service-tracker"/>
                                    </xsl:attribute>
                                </annotation>    
			</xsl:if>

                        <xsl:if test="count(@service-publisher) = 1">
				<annotation xsi:type="org.objectweb.dsrg.sofa.repository.model:ServicePublisher">
                                    <xsl:attribute name="type">
                                        <xsl:value-of select="@service-publisher"/>
                                    </xsl:attribute>
                                </annotation>    
			</xsl:if>


			<xsl:apply-templates />
		</org.objectweb.dsrg.sofa.repository.model:Frame>
	</xsl:template>

	<xsl:template match="/frame/behavior">
		<behavior>
			<xsl:attribute name="name">
				<xsl:value-of select="@type" />
			</xsl:attribute>
			<xsl:attribute name="value">
				<xsl:value-of select="text()" />
			</xsl:attribute>
		</behavior>
	</xsl:template>

	<xsl:template match="/frame/property">
		<property>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@type" />
		</property>
	</xsl:template>

	<xsl:template match="/frame/provides">
		<providedInterface>
			<xsl:call-template name="handleProvidesRequires" />
		</providedInterface>
	</xsl:template>

	<xsl:template match="/frame/requires">
		<requiredInterface>
			<xsl:call-template name="handleProvidesRequires" />
		</requiredInterface>
	</xsl:template>

	<xsl:template name="handleProvidesRequires">
		<xsl:copy-of select="@name" />
		<xsl:attribute name="communicationStyle">
			<xsl:choose>
				<xsl:when test="count(@comm-style) = 1">
					<xsl:value-of select="@comm-style" />
				</xsl:when>
				<xsl:otherwise>method_invocation</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>

		<!-- TODO; Type (normal, utility) -->
		<xsl:attribute name="connectionType"><!-- TODO: This is untested -->
			<xsl:choose>
				<xsl:when test="count(@type) = 1">
					<xsl:value-of select="@type" />
				</xsl:when>
				<xsl:otherwise>normal</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>

    <xsl:attribute name="isCollection">
      <xsl:choose>
        <xsl:when test="count(@collection) = 1">
          <xsl:value-of select="@collection" />
        </xsl:when>
        <xsl:otherwise>false</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>

		<interfaceType>
			<xsl:attribute name="href">
				<xsl:value-of select="@itf-type" />
			</xsl:attribute>
		</interfaceType>

		<xsl:apply-templates />
	</xsl:template>

	<xsl:template match="/frame/*[name()='requires' or name()='provides']/comm-feature">
		<communicationFeature>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@value" />
		</communicationFeature>
	</xsl:template>

	<xsl:template name="getInterfaceRef"> <!-- TODO: Fix checking. -->
		<!-- The reference of an interface is formed as '//@requiredInterface.' + index of the subcomponent -->
		<xsl:param name="interfaceName" />
		<xsl:choose>
			<xsl:when test="count(/frame/requires[@name=$interfaceName]) > 0">
				<xsl:text>//@requiredInterface.</xsl:text>
				<xsl:value-of select="count(/frame/requires[@name=$interfaceName]/preceding-sibling::requires)" />
			</xsl:when>

			<xsl:when test="count(/frame/provides[@name=$interfaceName]) > 0">
				<xsl:text>//@providedInterface.</xsl:text>
				<xsl:value-of select="count(/frame/provides[@name=$interfaceName]/preceding-sibling::provides)" />
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="/frame/*[name()='requires' or name()='provides']/factory">
		<annotation xsi:type="org.objectweb.dsrg.sofa.repository.model:Factory">
			<xsl:copy-of select="@method" />

			<xsl:if test="count(@return-itf) = 1"> <!-- TODO: Fix checking. -->
				<xsl:attribute name="returnInterface">
					<xsl:call-template name="getInterfaceRef">
						<xsl:with-param name="interfaceName">
							<xsl:value-of select="@return-itf" />
						</xsl:with-param>
					</xsl:call-template>
				</xsl:attribute>
			</xsl:if>

			<xsl:if test="count(@return-instance) = 1"> <!-- TODO: Fix checking. -->
				<xsl:attribute name="returnInstance">
					<xsl:value-of select="@return-instance" />
				</xsl:attribute>
			</xsl:if>
		</annotation>
	</xsl:template>
        
        <xsl:template match="/frame/*[name()='requires' or name()='provides']/service">
		<annotation xsi:type="org.objectweb.dsrg.sofa.repository.model:Service">
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@filter" />
			<xsl:copy-of select="@timeout" />
		</annotation>
	</xsl:template>
        
        <xsl:template match="/frame/*[name()='requires' or name()='provides']/publish">
		<annotation xsi:type="org.objectweb.dsrg.sofa.repository.model:Publish">
                    <xsl:apply-templates />
		</annotation>
	</xsl:template>

	<xsl:template match="//publish/property">
		<property>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@type" />
		</property>
	</xsl:template>



	<!-- ================================================================================================================= -->
	<!-- Architecture                                                                                                      -->
	<!-- ================================================================================================================= -->

	<xsl:template match="/architecture">
		<org.objectweb.dsrg.sofa.repository.model:Architecture>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@active" />
			<xsl:copy-of select="@singleton" />
			<xsl:attribute name="implementation">
				<xsl:value-of select="@impl" />
			</xsl:attribute>

			<implements>
				<xsl:attribute name="href">
					<xsl:value-of select="@frame" />
				</xsl:attribute>
			</implements>
			<info name="language">
				<xsl:attribute name="value">
					<xsl:value-of select="@lang" />
				</xsl:attribute>
			</info>

			<xsl:apply-templates />
		</org.objectweb.dsrg.sofa.repository.model:Architecture>
	</xsl:template>

	<xsl:template name="getSubCompRef">
		<!-- The reference of a subcomponent is formed by '//@subcomponent.' + index of the subcomponent -->
		<xsl:param name="subCompName" />

    <!-- numbers of occurances: -->
    <xsl:variable name="subcomponents" select="count(/architecture/sub-comp[@name=$subCompName])" />
    <!-- report too many occurances / no occurance -->
    <xsl:if test="$subcomponents &gt; 1">
      <xsl:message terminate="yes">
        Too many subcomponents named '<xsl:value-of select="$subCompName" />'.
      </xsl:message>
    </xsl:if>
    <xsl:if test="$subcomponents &lt; 1">
      <xsl:message terminate="yes">
        No subcomponent named '<xsl:value-of select="$subCompName" />'.
      </xsl:message>
    </xsl:if>
    <!-- form the reference -->
		<xsl:text>//@subcomponent.</xsl:text>
		<xsl:value-of select="count(/architecture/sub-comp[@name=$subCompName]/preceding-sibling::sub-comp)" />
	</xsl:template>

	<xsl:template match="/architecture/sub-comp">
		<subcomponent>
			<xsl:copy-of select="@name" />
      <xsl:attribute name="isCollection">
        <xsl:choose>
          <xsl:when test="count(@collection) = 1">
            <xsl:value-of select="@collection" />
          </xsl:when>
          <xsl:otherwise>false</xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>

			<instantiatesFrame>
				<xsl:attribute name="href">
					<xsl:value-of select="@frame" />
				</xsl:attribute>
			</instantiatesFrame>

			<xsl:if test="count(@arch) = 1">
				<instantiatesArchitecture>
					<xsl:attribute name="href">
						<xsl:value-of select="@arch" />
					</xsl:attribute>
				</instantiatesArchitecture>
			</xsl:if>

		</subcomponent>
	</xsl:template>

	<xsl:template match="/architecture/connection">
		<connection>
			<xsl:apply-templates />
		</connection>
	</xsl:template>

	<xsl:template match="/architecture/connection/endpoint[count(@sub-comp) = 1]">
		<endpoint xsi:type="org.objectweb.dsrg.sofa.repository.model:SubcomponentInterfaceEndpoint">
			<xsl:attribute name="interfaceName">
				<xsl:value-of select="@itf" />
			</xsl:attribute>

			<xsl:attribute name="subcomponent">
				<xsl:call-template name="getSubCompRef">
					<xsl:with-param name="subCompName">
						<xsl:value-of select="@sub-comp" />
					</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>
		</endpoint>
	</xsl:template>

	<xsl:template match="/architecture/connection/endpoint[count(@sub-comp) = 0]">
		<endpoint xsi:type="org.objectweb.dsrg.sofa.repository.model:ComponentInterfaceEndpoint">
			<xsl:attribute name="interfaceName">
				<xsl:value-of select="@itf" />
			</xsl:attribute>
		</endpoint>
	</xsl:template>

	<xsl:template match="/architecture/mapped-property">
		<mappedProperty>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@type" />

			<xsl:attribute name="instance">
				<xsl:call-template name="getSubCompRef">
					<xsl:with-param name="subCompName">
						<xsl:value-of select="@sub-comp" />
					</xsl:with-param>
				</xsl:call-template>
			</xsl:attribute>

			<xsl:attribute name="subcomponentPropertyName">
				<xsl:value-of select="@sub-comp-property" />
			</xsl:attribute>
		</mappedProperty>
	</xsl:template>
	
	<xsl:template match="/architecture/property-reference">
		<propertySets>
			<xsl:attribute name="href">
				<xsl:value-of select="@set" />
			</xsl:attribute>
		</propertySets>
	</xsl:template>

	<xsl:template match="/architecture/property">
		<property>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@type" />
		</property>
	</xsl:template>
	<xsl:template match="/architecture/dependency">
  </xsl:template>

	<!-- ================================================================================================================= -->
	<!-- Assembly                                                                                                          -->
	<!-- ================================================================================================================= -->

	<xsl:template match="/assembly">
		<org.objectweb.dsrg.sofa.repository.model:AssemblyDescriptor>
			<xsl:copy-of select="@name" />

			<topLevelInstance>
				<architecture>
					<xsl:attribute name="href">
						<xsl:value-of select="@top-level-arch" />
					</xsl:attribute>
				</architecture>
				<xsl:apply-templates />
			</topLevelInstance>

		</org.objectweb.dsrg.sofa.repository.model:AssemblyDescriptor>
	</xsl:template>

	<xsl:template match="/assembly//subcomponent">
		<subcomponent>
			<xsl:copy-of select="@name" />
			<instance>
				<architecture>
					<xsl:attribute name="href">
						<xsl:value-of select="@arch" />
					</xsl:attribute>
				</architecture>
				<xsl:apply-templates />
			</instance>
		</subcomponent>
	</xsl:template>

	<xsl:template match="/assembly//dynamic-instance">
		<dynamicInstance>
			<architecture>
				<xsl:attribute name="href">
					<xsl:value-of select="@arch" />
				</xsl:attribute>
			</architecture>
			<xsl:apply-templates />
		</dynamicInstance>
	</xsl:template>

	<!-- ================================================================================================================= -->
	<!-- Code bundle                                                                                                       -->
	<!-- ================================================================================================================= -->

	<xsl:template match="/code-bundle">
		<org.objectweb.dsrg.sofa.repository.model:CodeBundle>
		<xsl:attribute name="language">
			<xsl:value-of select="@lang" />
		</xsl:attribute>
      <xsl:attribute name="thirdParty">
        <xsl:choose>
          <xsl:when test="count(@third-party) = 1">
            <xsl:value-of select="@third-party" />
          </xsl:when>
          <xsl:otherwise>false</xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
			<xsl:copy-of select="@name" />
			<xsl:apply-templates />
		</org.objectweb.dsrg.sofa.repository.model:CodeBundle>
	</xsl:template>
	

	<xsl:template match="/code-bundle/dependency">
		<dependencies>
      <xsl:attribute name="href">
				<xsl:value-of select="@bundle" />
			</xsl:attribute>
		</dependencies>
	</xsl:template>
	
	<!-- ================================================================================================================= -->
	<!-- Property set                                                                                                      -->
	<!-- ================================================================================================================= -->

	<xsl:template match="/property-set">
		<org.objectweb.dsrg.sofa.repository.model:PropertySet>
			<xsl:copy-of select="@name" />
			<xsl:apply-templates />
		</org.objectweb.dsrg.sofa.repository.model:PropertySet>
	</xsl:template>
	

	<xsl:template match="/property-set/property">
		<property>
			<xsl:copy-of select="@name" />
			<xsl:copy-of select="@type" />
		</property>
	</xsl:template>

	<!-- ================================================================================================================= -->
	<!-- MicroInterfaceType                                                                                                -->
	<!-- ================================================================================================================= -->
	<xsl:template match="/mitf-type">
		<org.objectweb.dsrg.sofa.repository.model:MicroInterfaceType>
      <xsl:call-template name="handleInterfaceType" />
		</org.objectweb.dsrg.sofa.repository.model:MicroInterfaceType>
	</xsl:template>
	<xsl:template match="/mitf-type/dependency">
  </xsl:template>

	<!-- ================================================================================================================= -->
	<!-- MicroComponent                                                                                                    -->
	<!-- ================================================================================================================= -->

  <xsl:template match="/mcomponent">
    <xsl:if test="count(/mcomponent/delegates) &gt; 1"><!-- simple error check -->
      <xsl:message terminate="yes">
        Too many occurances of element &lt;delegates&gt;.
      </xsl:message>
    </xsl:if>
    <org.objectweb.dsrg.sofa.repository.model:MicroComponent>
      <xsl:copy-of select="@name" />
      <xsl:if test="delegates/@provided">
        <xsl:attribute name="delegatesProvided">
          <xsl:value-of select="delegates/@provided" />
        </xsl:attribute>
      </xsl:if>
      <xsl:if test="delegates/@required">
        <xsl:attribute name="delegatesRequired">
          <xsl:value-of select="delegates/@required" />
        </xsl:attribute>
      </xsl:if>
      <content>
        <xsl:attribute name="implementation">
          <xsl:value-of select="@impl" />
        </xsl:attribute>
        <xsl:choose>
          <xsl:when test="@impl-type = 'generator'">
           <xsl:attribute name="xsi:type">org.objectweb.dsrg.sofa.repository.model:MicroContentGenerator</xsl:attribute>
          </xsl:when>
          <xsl:when test="@impl-type = 'class'">
            <xsl:attribute name="xsi:type">org.objectweb.dsrg.sofa.repository.model:MicroContentClass</xsl:attribute>
          </xsl:when>
          <xsl:otherwise>
            <xsl:message terminate="yes">
              Unexpected micro content type (attribute 'impl-type' of element &lt;mcomponent&gt;).
              Expected values are 'generator' or 'class'.
            </xsl:message>
          </xsl:otherwise>
        </xsl:choose>
      </content>
      <xsl:apply-templates />
    </org.objectweb.dsrg.sofa.repository.model:MicroComponent>
  </xsl:template>
  <xsl:template match="/mcomponent/requires">
    <requiredInterface>
			<xsl:call-template name="handleMuProvidesRequires" />
    </requiredInterface>
  </xsl:template>
  <xsl:template match="/mcomponent/provides">
    <providedInterface>
			<xsl:call-template name="handleMuProvidesRequires" />
    </providedInterface>
  </xsl:template>
  <xsl:template match="/mcomponent/delegates">
    <xsl:if test="@mitf-type">
      <delegatedInterfaceType>
        <xsl:call-template name="handleMuItfTypeRef" />
      </delegatedInterfaceType>
    </xsl:if>
  </xsl:template>
	<xsl:template name="handleMuProvidesRequires">
		<xsl:copy-of select="@name" />
    <interfaceType>
      <xsl:call-template name="handleMuItfTypeRef" />
    </interfaceType>
  </xsl:template>
	<xsl:template name="handleMuItfTypeRef">
    <xsl:attribute name="href">
      <xsl:value-of select="@mitf-type" />
	  </xsl:attribute>
  </xsl:template>

  <xsl:template match="/mcomponent/dependency">
  </xsl:template>

	<!-- ================================================================================================================= -->
	<!-- Aspect                                                                                                            -->
	<!-- ================================================================================================================= -->

  <xsl:template match="/aspect">
    <org.objectweb.dsrg.sofa.repository.model:Aspect>
      <xsl:copy-of select="@*" />
      <xsl:apply-templates />
    </org.objectweb.dsrg.sofa.repository.model:Aspect>
  </xsl:template>

  <xsl:template match="/aspect/component-select">
    <componentSelect>
      <xsl:copy-of select="@query" />
      <xsl:apply-templates /> 
    </componentSelect>
  </xsl:template>

  <xsl:template match="/aspect/component-select/ctrl-itf">
    <controlInterface>
      <xsl:copy-of select="@name" />
      <xsl:attribute name="instance">
        <xsl:call-template name="getMuInstanceRef">
          <xsl:with-param name="muInstanceName">
            <xsl:value-of select="@inst" />
          </xsl:with-param>
        </xsl:call-template>
      </xsl:attribute>
      <interfaceType>
        <xsl:attribute name="href">
          <xsl:value-of select="@mitf-type" />
        </xsl:attribute>
      </interfaceType>
    </controlInterface>
  </xsl:template>

  <xsl:template match="/aspect/component-select/interface-select">
    <interfaceSelect>
      <xsl:copy-of select="@query" />
      <xsl:apply-templates /> 
    </interfaceSelect>
  </xsl:template>

  <xsl:template match="/aspect/component-select//instance">
    <!-- the match pattern is too weak for strict checking,             -->
    <!-- but we expect that the document already conforms to the schema -->
    <instance>
      <xsl:call-template name="handleMicroComponentInstance" />
    </instance>
  </xsl:template>

  <xsl:template match="/aspect/component-select/interface-select/delegating-inst">
    <delegatingInstance>
      <xsl:call-template name="handleMicroComponentInstance" />
    </delegatingInstance>
  </xsl:template>

  <xsl:template name="handleMicroComponentInstance">
    <xsl:copy-of select="@name" />
    <microcomponent>
      <xsl:attribute name="href">
        <xsl:value-of select="@mcomp" />
      </xsl:attribute>
    </microcomponent>
    <xsl:apply-templates />
  </xsl:template>

  <xsl:template match="/aspect/component-select//param">
    <parameter>
      <xsl:copy-of select="@*" />
    </parameter>
  </xsl:template>

  <xsl:template match="/aspect/component-select//binding">
    <!-- the match pattern is too weak for strict checking,             -->
    <!-- but we expect that the document already conforms to the schema -->
    <binding>
      <xsl:apply-templates />
    </binding>
  </xsl:template>

  <xsl:template match="/aspect/component-select//binding/server">
    <server>
      <xsl:call-template name="handleMicroBindingEndPoint" />
    </server>
  </xsl:template>

  <xsl:template match="/aspect/component-select//binding/client">
    <client>
      <xsl:call-template name="handleMicroBindingEndPoint" />
    </client>
  </xsl:template>

  <xsl:template name="handleMicroBindingEndPoint">
    <xsl:if test="@inst">
      <xsl:attribute name="componentName">
        <xsl:value-of select="@inst" />
      </xsl:attribute>
    </xsl:if>
    <xsl:if test="@itf">
      <xsl:attribute name="interfaceName">
        <xsl:value-of select="@itf" />
      </xsl:attribute>
    </xsl:if>
  </xsl:template>

	<xsl:template name="getMuInstanceRef"> <!-- form microComponentInstance reference -->
		<xsl:param name="muInstanceName" />
    <!-- numbers of occurances: -->
    <xsl:variable name="cSelectInstances" select="count(/aspect/component-select/instance[@name=$muInstanceName])" />
    <xsl:variable name="cSelectISelectInstances" select="count(/aspect/component-select/interface-select/instance[@name=$muInstanceName])" />
    <xsl:variable name="cSelectISelectDelegInsts" select="count(/aspect/component-select/interface-select/delegating-inst[@name=$muInstanceName])" />
    <xsl:variable name="totalInstances" select="$cSelectInstances + $cSelectISelectInstances + $cSelectISelectDelegInsts" />

    <!-- report too many occurances / no occurance -->
    <xsl:if test="$totalInstances &gt; 1">
      <xsl:message terminate="yes">
        Too many microcomponent instances named '<xsl:value-of select="$muInstanceName" />'.
      </xsl:message>
    </xsl:if>
    <xsl:if test="$totalInstances &lt; 1">
      <xsl:message terminate="yes">
        No microcomponent instance named '<xsl:value-of select="$muInstanceName" />'.
      </xsl:message>
    </xsl:if>

    <!-- form the reference in one of three possible ways -->
    <xsl:choose>
      <xsl:when test="$cSelectInstances = 1">
        <!-- component-select/instance: the reference is formed like
             '//@componentSelect.I/@instance.J' -->
    		<xsl:text>//@componentSelect.</xsl:text>
  	  	<xsl:value-of select="count(/aspect/component-select/instance[@name=$muInstanceName]/../preceding-sibling::component-select)" />
        <xsl:text>/@instance.</xsl:text>
        <xsl:value-of select="count(/aspect/component-select/instance[@name=$muInstanceName]/preceding-sibling::instance)" />
      </xsl:when>
      <xsl:when test="$cSelectISelectInstances = 1">
        <!-- component-select/interface-select/instance: the reference is formed like
             '//@componentSelect.I/@interfaceSelect.J/@instance.K' -->
  	  	<xsl:text>//@componentSelect.</xsl:text>
  	  	<xsl:value-of select="count(/aspect/component-select/interface-select/instance[@name=$muInstanceName]/../../preceding-sibling::component-select)" />
  	  	<xsl:text>/@interfaceSelect.</xsl:text>
  	  	<xsl:value-of select="count(/aspect/component-select/interface-select/instance[@name=$muInstanceName]/../preceding-sibling::interface-select)" />
        <xsl:text>/@instance.</xsl:text>
        <xsl:value-of select="count(/aspect/component-select/interface-select/instance[@name=$muInstanceName]/preceding-sibling::instance)" />
      </xsl:when>
      <xsl:when test="$cSelectISelectDelegInsts = 1">
        <!-- component-select/interface-select/instance: the reference is formed like
             '//@componentSelect.I/@interfaceSelect.J/@delegatingInstance.K' -->
  	  	<xsl:text>//@componentSelect.</xsl:text>
  	  	<xsl:value-of select="count(/aspect/component-select/interface-select/delegating-inst[@name=$muInstanceName]/../../preceding-sibling::component-select)" />
  	  	<xsl:text>/@interfaceSelect.</xsl:text>
  	  	<xsl:value-of select="count(/aspect/component-select/interface-select/delegating-inst[@name=$muInstanceName]/../preceding-sibling::interface-select)" />
        <xsl:text>/@delegatingInstance</xsl:text>
      </xsl:when>
    </xsl:choose>
	</xsl:template>

	<!-- ================================================================================================================= -->
	<!-- Error reporting                                                                                                   -->
	<!-- ================================================================================================================= -->
	<xsl:template match="*|/" priority="-1">
		<xsl:message terminate="yes">
			Unexpected element found:
			<xsl:for-each select="ancestor-or-self::*">
				<xsl:value-of select="name(.)" />
				/
			</xsl:for-each>
		</xsl:message>
	</xsl:template>

	<xsl:template match="text()" priority="-1">
		<xsl:message terminate="yes">
			Unexpected text node: '
			<xsl:copy-of select="." />
			' Path is
			<xsl:for-each select="ancestor-or-self::*">
				<xsl:value-of select="name(.)" />
				/
			</xsl:for-each>
		</xsl:message>
	</xsl:template>

	<!--
		<xsl:template match="@*" priority="-1">
		<xsl:message terminate="yes">
		Unexpected attribute node: '<xsl:copy-of select="name(.)"/>'
		Path is <xsl:for-each select="ancestor-or-self::*"><xsl:value-of select="name(.)"/> / </xsl:for-each>
		</xsl:message>
		</xsl:template>
	-->

</xsl:stylesheet>
