/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import java.util.Map;

import javax.script.ScriptEngine;

import org.objectweb.dsrg.sofa.SOFAException;
import org.objectweb.dsrg.sofa.repository.model.CodeBundle;

/**
 * Interface responsible for correct scripting engine initialization for
 * purposes of {@link MIScriptInvoker}.
 * 
 * @author Jaroslav Keznikl
 */
public interface EngineInitializer {
	
	/**
	 * Creates and initializes a new instance of the {@link ScriptEngine}
	 * according to the given script language.
	 * 
	 * <p>Inserts a {@link SOFAScriptImporter} and {@link SOFAPythonImporter}
	 * references to the engine context. </p>
	 * 
	 * <p>Inserts all required interface references to the new scripting engine
	 * under the given names</p>
	 * 
	 * <p>Set up the given {@link ClassLoader} to be used by the new
	 * scripting engine.</p>
	 * 
	 * @param loader	The class loader to be used by the new scripting engine.
	 * @param scriptCode	The script code of the component architecture implementation.
	 * @param codeBundle	The associated code bundle of the component architecture.
	 * @param requiredIfaces	References to all required interfaces.
	 * @return Newly created and initialized scripting engine.
	 * @throws SOFAException
	 */
	public ScriptEngine createScriptingEngine(ClassLoader loader, String scriptCode, CodeBundle codeBundle, Map<String, Object> requiredIfaces) throws SOFAException;

}
