/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import java.io.StringReader;
import java.util.Map;

import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;

import org.objectweb.dsrg.sofa.SOFAException;
import org.objectweb.dsrg.sofa.repository.model.CodeBundle;

/**
 * {@link EngineInitializer} implementation for the Mozilla Rhino JavaScript engine.
 * 
 * @author Jaroslav Keznikl 
 */
public class JavascriptEngineInitializer extends AbstractEngineInitializer implements EngineInitializer {
	
	/** The name of the supported language. */
	public static final String LANGUAGE_NAME = "ECMAScript";
	
	/** The associated script engine manager. */
	private ScriptEngineManager scriptEngineMgr;
	
	/** Initializes the inner fields. */
	public JavascriptEngineInitializer(ScriptEngineManager mgr) {
		scriptEngineMgr = mgr;
	}
	
	
	/**
	 * {@link EngineInitializer#createScriptingEngine(ClassLoader, String, CodeBundle, Map)}
	 * 
	 * The class loader is configured using the context class loader of current thread.
	 */
	public ScriptEngine createScriptingEngine(ClassLoader loader,
			String scriptCode, CodeBundle codeBundle,
			Map<String, Object> requiredIfaces) throws SOFAException {
				
		Thread currentThread = Thread.currentThread();
        
		ClassLoader oldClassloader = currentThread.getContextClassLoader();
		currentThread.setContextClassLoader(loader);
		ScriptEngine jsEngine;
		
		try {
			jsEngine = scriptEngineMgr.getEngineByName(LANGUAGE_NAME);
		} finally {
			currentThread.setContextClassLoader(oldClassloader);
		}
		
		
		if (jsEngine == null)
			throw new SOFAException("Scripting engine couldn't be created.");
		
		try {
			SOFAScriptImporter scriptImporter = new SOFAScriptImporterImpl(jsEngine, codeBundle);
			jsEngine.put(SCRIPT_IMPORTER_VAR_NAME, scriptImporter);
			
			jsEngine.eval(new StringReader(scriptCode)); 
		} catch (javax.script.ScriptException e) {
			throw new SOFAException(e);
		}		
		
		for (String requiredName: requiredIfaces.keySet()) {
			 jsEngine.put(requiredName, requiredIfaces.get(requiredName));
		}
		
		return jsEngine;
	}

}
