/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2008  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import java.util.Map;
import java.util.Set;

import org.objectweb.dsrg.sofa.SOFAException;
import org.objectweb.dsrg.sofa.SOFAInterfaceType;
import org.objectweb.dsrg.sofa.microarchitecture.ComponentDescription;
import org.objectweb.dsrg.sofa.microarchitecture.DelegationChain;
import org.objectweb.dsrg.sofa.microarchitecture.MIComponent;
import org.objectweb.dsrg.sofa.microarchitecture.SOFAMicroComponent;
import org.objectweb.dsrg.sofa.microarchitecture.SOFAMicroInterceptor;

/**
 * Implementation of mircocomponent that intercepts the {@link MIComponent} control interface. 
 * Notifies the {@link MScriptInvoker} microcomponent whenever the
 * component content is about to be replaced.
 * 
 * @author Jaroslav Keznikl
 */
public class MComponentProxy implements SOFAMicroComponent, SOFAMicroInterceptor, MIComponent {
  
	MIContentListener listener;
	MIComponent endpoint;
	
	public void init(ComponentDescription component,
			Map<String, String> parameters) throws SOFAException {		
	}

    public Object getProvided (String name) throws SOFAException {
    	return null;
    }


	/**
	 * Sets required interface (listener).
	 * 
	 */
	public void setRequired(String name, Object ref) throws SOFAException {		
		if (name != null && name.equals("listener")) {
			if (ref instanceof MIContentListener) {
				listener = (MIContentListener) ref;
			} else {
				throw new SOFAException("Unable to set required interface 'listener'. Incompatible type.");
			}
		}
		// we silently ignore interfaces we don't need
	}
	
	/**
	 * Notify the {@link MIContentListener} before setting the new content.
	 */
	public void setContent(Object o) {
		listener.contentAboutToBeChanged(o);
		endpoint.setContent(o);
	}
	
	public Object getDelegatedProvided() throws SOFAException {
		return this;
	}
	
	public void setDelegatedRequired(Object ref) throws SOFAException {		
		endpoint = (MIComponent) ref;		
	}

	public DelegationChain getDelegationChain(SOFAInterfaceType type,
			String name) throws SOFAException {
		return endpoint.getDelegationChain(type, name);		
	}

	public Set<DelegationChain> getDelegationChains() {
		return endpoint.getDelegationChains();
	}

	public Set<String> getControlInterfaces() {
		return endpoint.getControlInterfaces();
	}


	public Object getControlInterface(String name) {
		return endpoint.getControlInterface(name);
	}


	public Object getContent() {
		return endpoint.getContent();
	}


	public Object getFrame() {
		return endpoint.getFrame();
	}

}
