/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import org.objectweb.dsrg.sofa.SOFAException;

/**
 * Interface that ensures scripting engine from other aspects.
 * 
 * This interface is intended to be used internally by other aspects and not directly.
 *  
 * @author Jaroslav Keznikl
 */
public interface MIScript {
	
	/**
	 * Executes given script text. 
	 * @param text
	 * @throws SOFAException
	 */
	void evaluateScriptText(String text) throws SOFAException;	
	
	/**
	 * Resets the script engine to the initial state
	 * @throws SOFAException
	 */
	void reset() throws SOFAException;
	
	/**
	 * Sets a key/value pair in the state of the scripting engine.
	 * @param name The name of the value to add.
	 * @param value The value of the value to add.
	 * @throws SOFAException 
	 */
	void setValue(String name, Object value) throws SOFAException;
	
	/**
	 * Retrieves a value set in the state of the scripting engine. 
	 * The value might be one which was set using setValue or some 
	 * other value in the state of the scripting engine, depending 
	 * on the implementation
	 * @param name The key whose value is to be returned.
	 * @return The value associated with the given key.
	 * @throws SOFAException 
	 */
	Object getValue(String name) throws SOFAException;
}
