/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import javax.script.ScriptException;

import org.objectweb.dsrg.sofa.SOFAException;
import org.ow2.dsrg.sofa.aspects.script.NotScriptException;

/**
 * Interface for script function invocation.
 * @author Jaroslav Keznikl
 */
public interface MIScriptInvoker {

	/**
	 * Invoke target script function . 
	 * 
	 * Invokes target script function with given arguments and return 
	 * the return value or null.
	 * 
	 * @param method	The target function name.
	 * @param params	The parameters of the call.
	 * @return 	The script return value.
	 * @throws NotScriptException	If the associated component is not a scripted component.
	 * @throws NoSuchMethodException	If the target function was not found.
	 * @throws ScriptException	If the script execution failed.
	 * @throws SOFAException	If the scripting engine is not initialized.
	 */
	Object invoke(String method, Object[] params) throws SOFAException,
			NotScriptException, ScriptException, NoSuchMethodException;

}
