/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2008  DSRG, Charles University in Prague
 *                          http://dsrg.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import org.objectweb.dsrg.sofa.SOFAException;

/**
 * Interface for UpdateScript control interface. Is part of the UpdateScript aspect.
 * 
 * <p> This interface ensures the script engine manipulation and code updates.</p> 
 * 
 * @author Jaroslav Keznikl
 */
public interface MIUpdateScript {
		
	/**
	 * Executes given script text. 
	 * @param text
	 * @throws SOFAException
	 */
	void evaluateScriptText(String text) throws SOFAException;

	/**
	 * Executes given script text.
	 * 
	 * @param text
	 * @param append True if the given script has to be appended to the script
	 *            	 history. The history would be reset otherwise.
	 * @throws SOFAException
	 */
	void evaluateScriptText(String text, Boolean append) throws SOFAException;	
	
	/**
	 * Resets the script engine to the initial state
	 * @throws SOFAException
	 */
	void reset() throws SOFAException;
	
	/**
	 * Sets a key/value pair in the state of the scripting engine.
	 * @param name The name of the value to add.
	 * @param value The value of the value to add.
	 * @throws SOFAException 
	 */
	void setValue(String name, Object value) throws SOFAException;
	
	/**
	 * Retrieves a value set in the state of the scripting engine. 
	 * The value might be one which was set using setValue or some 
	 * other value in the state of the scripting engine, depending 
	 * on the implementation
	 * @param name The key whose value is to be returned.
	 * @return The value associated with the given key.
	 * @throws SOFAException 
	 */
	Object getValue(String name) throws SOFAException;
	
	/**
	 * Returns a concatenation of all scripts evaluated by {@link evaluateScriptText} plus the original code.
	 * @return Concatenation of all evaluated scripts plus the original code. 
	 */
	String getCurrentScript();
	
	/**
	 * Returns the original component implementation code.
	 * @return The original component code. 
	 */
	String getOriginalScript();

	/**
	 * Stores current script into repository. 
	 * 
	 * Stores current script into repository in the associated implementation code 
	 * file so that the application would be in the same state after restart.
	 * @throws SOFAException 
	 */
	void storeCurrentScript() throws SOFAException;
}
