/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import org.objectweb.dsrg.sofa.SOFAException;
import java.util.Map;
import org.objectweb.dsrg.sofa.microarchitecture.ComponentDescription;
import org.objectweb.dsrg.sofa.microarchitecture.MIComponent;
import org.objectweb.dsrg.sofa.microarchitecture.SOFAMicroComponent;
import org.objectweb.dsrg.sofa.microarchitecture.SOFAMicroInterceptor;
import org.objectweb.dsrg.sofa.bootstrap.MIInterceptor;

/**
 * Implementation of mircocomponent that intercepts the {@link MIScript} control interface. 
 * 
 * Redirects all requests to the {@link MScriptInvoker} microcomponent.
 * 
 * @author Jaroslav Keznikl
 */
public class MScriptProxy implements SOFAMicroComponent, SOFAMicroInterceptor, MIScript {

	/** The required interface reference. */
	MIScript scriptIfaceImpl;
	
	public void init(ComponentDescription component,
			Map<String, String> parameters) throws SOFAException {		
	}

	
	/**
	 * {@link MIScript#evaluateScriptText(String)}
	 */
	public void evaluateScriptText(String text) throws SOFAException {
		scriptIfaceImpl.evaluateScriptText(text);
	}	

	/**
	 * {@link MIScript#reset()}
	 */
	public void reset() throws SOFAException {
		scriptIfaceImpl.reset();		
	}
	
	/**
	 * {@link MIScript#getValue(String)}
	 */
	public Object getValue(String name) throws SOFAException {
		return scriptIfaceImpl.getValue(name);
	}

	/**
	 * {@link MIScript#setValue(String, Object)}
	 */
	public void setValue(String name, Object value) throws SOFAException {
		scriptIfaceImpl.setValue(name, value);		
	}

	/** No provided interfaces available. */
    public Object getProvided (String name) throws SOFAException {
    	return null;
    }


	/**
	 * Sets the required interface (script).
	 */
	public void setRequired(String name, Object ref) throws SOFAException {		
		if (name != null && name.equals("script")) {
			if (ref instanceof MIScript) {
				scriptIfaceImpl = (MIScript) ref;
			} else {
				throw new SOFAException(
						"Unable to set required interface 'script'. Incompatible type.");
			}
		}
		// we silently ignore interfaces we dont need
	}
	
	public Object getDelegatedProvided() throws SOFAException {
		return this;
	}


	/** 
	 * The microcomponent doesn't delegate any required interfaces as it's
	 * an control interface implementation.
	 */
	public void setDelegatedRequired(Object ref) throws SOFAException {
		// ignore incoming calls		
	}




}
