/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import java.util.Map;

import org.apache.log4j.Logger;
import org.objectweb.dsrg.sofa.SOFAException;
import org.objectweb.dsrg.sofa.microarchitecture.ComponentDescription;
import org.objectweb.dsrg.sofa.microarchitecture.SOFAMicroComponent;
import org.objectweb.dsrg.sofa.microarchitecture.SOFAMicroInterceptor;

/**
 * Implementation of the {@link MIUpdateScript} control interface, redirects
 * calls to the {@link MUpdateScriptImpl} microcomponent.
 * 
 * @author Jaroslav Keznikl
 */
public class MUpdateScriptProxy implements SOFAMicroComponent,
		SOFAMicroInterceptor, MIUpdateScript {

	final static private Logger logger = Logger.getLogger(MUpdateScriptProxy.class);
	
	/** Reference to the {@link MUpdateScriptImpl} microcomponent. */
	MIUpdateScript updateScriptIfaceImpl;	
	
	/**
	 * {@link MIUpdateScript#storeCurrentScript()}
	 */
	public void storeCurrentScript() throws SOFAException {
		updateScriptIfaceImpl.storeCurrentScript();
	}
	
	
	
	/** 
	 * {@link MIUpdateScript#evaluateScriptText(String, Boolean)}
	 */
	public void evaluateScriptText(String text, Boolean append) throws SOFAException {
		updateScriptIfaceImpl.evaluateScriptText(text, append);		
	}

	/** 
	 * {@link MIUpdateScript#evaluateScriptText(String)}
	 */
	public void evaluateScriptText(String text) throws SOFAException {
		updateScriptIfaceImpl.evaluateScriptText(text);
	}

	/** 
	 * {@link MIUpdateScript#reset()}
	 */
	public void reset() throws SOFAException {
		updateScriptIfaceImpl.reset();
	}

	/** 
	 * {@link MIUpdateScript#getValue(String)}
	 */
	public Object getValue(String name) throws SOFAException {
		return updateScriptIfaceImpl.getValue(name);
	}

	/** 
	 * {@link MIUpdateScript#setValue(String, Object)}
	 */
	public void setValue(String name, Object value) throws SOFAException {
		updateScriptIfaceImpl.setValue(name, value);
	}

	/** 
	 * {@link MIUpdateScript#getCurrentScript()}
	 */
	public String getCurrentScript() {
		return updateScriptIfaceImpl.getCurrentScript();
	}

	/** 
	 * {@link MIUpdateScript#getOriginalScript()}
	 */
	public String getOriginalScript() {
		return updateScriptIfaceImpl.getOriginalScript();
	}

	/** 
	 * No provided interfaces, ignored.
	 */
	public Object getProvided(String name) throws SOFAException {
		return null;
	}

	/**
	 * Sets required the interface (update-script). 
	 */
	public void setRequired(String name, Object ref) throws SOFAException {
		if (name != null && name.equals("update-script")) {
			if (ref instanceof MIUpdateScript) {
				updateScriptIfaceImpl = (MIUpdateScript) ref;
			} else {
				throw new SOFAException("Unable to set required interface 'update-script'. Incompatible type.");
			}
		}
		// we silently ignore interfaces we don't need
	}
	
	public Object getDelegatedProvided() throws SOFAException {
		return this;
	}

	/** 
	 * The microcomponent doesn't delegate any required interfaces as it's
	 * an control interface implementation.
	 */
	public void setDelegatedRequired(Object ref) throws SOFAException {
		// ignore incoming calls
	}



	public void init(ComponentDescription component,
			Map<String, String> parameters) throws SOFAException {
		
	}


	
}
