/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import java.io.IOException;

import org.objectweb.dsrg.sofa.SOFAException;

/**
 * Interface for Python-specific module importing support.
 * 
 * <p>Enables a script developer to download and add the target code bundle in the
 * engine's system path so that the scripting engine can search this bundle for
 * python modules.</p>
 * 
 * @author Jaroslav Keznikl
 */
public interface SOFAPythonImporter {
	
	/**
	 * Adds the current code bundle and all dependent bundles in the engine's system path.
	 * @throws IOException
	 * @throws SOFAException
	 */
	void loadAllDependencies() throws IOException, SOFAException;
	
	/**
	 * Adds the current code bundle in the engine's system path.
	 * @throws IOException
	 * @throws SOFAException
	 */
	void loadCurrentCodeBundle() throws IOException, SOFAException;
	
	/**
	 * Adds the given code bundle (with the 'current' tag) in the engine's system path.
	 * @param cbName the target bundle name
	 * @throws IOException
	 * @throws SOFAException
	 */
	void loadCodeBundle(String cbName) throws IOException, SOFAException;
	
	/**
	 * Adds the given code bundle in the engine's system path.
	 * Either @a version or @a tag must not be null. 
	 * @param cbName	the target bundle name
	 * @param cbVersion	the target bundle version
	 * @param cbTag		the target bundle tag
	 * @throws IOException
	 * @throws SOFAException
	 */
	void loadCodeBundle(String cbName, String cbVersion, String cbTag) throws IOException, SOFAException;
}
