/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.aspects.script;

import org.objectweb.dsrg.sofa.SOFAException;

/**
 * Interface for language-independent script module importing support.
 * 
 * <p>Enables a script developer to evaluate an arbitrary script file from 
 * current code bundle and all its dependencies in the current engine. 
 * This mechanism is independent on the scripting language.</p>
 * 
 * @author Jaroslav Keznikl
 */
public interface SOFAScriptImporter {
	/**
	 * Evaluates the given file. Searches the current code bundle and all its dependencies.
	 * @param fileName	relative path of the file in the associated code bundle
	 * @throws SOFAException
	 */
	void importFile(String fileName) throws SOFAException;
	
	/**
	 * Evaluates the given file. Searches the current code bundle only.
	 * @param fileName relative path of the file in the current code bundle
	 * @throws SOFAException
	 */
	void importFromCurrent(String fileName) throws SOFAException;
	
	/**
	 * Evaluates the given file. Searches the given code bundle only.
	 * The bundle is specified using its name, the 'current' tag is used implicitly.
	 * @param fileName	relative path of the file in the associated code bundle	
	 * @param codeBundle	name of the target code bundle
	 * @throws SOFAException
	 */
	void importFile(String fileName, String codeBundle) throws SOFAException;
	
	/**
	 * Evaluates the given file. Searches the given code bundle only.
	 * The bundle is specified using its name, tag and version. 
	 * Either bundle version or tag must not be null.
	 * @param fileName		relative path of the file in the associated code bundle
	 * @param codeBundle	name of the target code bundle
	 * @param cbVersion	version of the target code bundle
	 * @param cbTag		tag of the target code bundle
	 * @throws SOFAException
	 */
	void importFile(String fileName, String codeBundle, String cbVersion, String cbTag) throws SOFAException;  
	
}
