/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */
package org.ow2.dsrg.sofa.examples.script.logdemo.arch;
import java.util.Map;

import org.objectweb.dsrg.sofa.SOFAClient;
import org.objectweb.dsrg.sofa.SOFALifecycle;
import org.objectweb.dsrg.sofa.SOFAUpdatable;
import org.ow2.dsrg.sofa.examples.script.logdemo.cbundle.Common.LogDisabledException;
import org.ow2.dsrg.sofa.examples.script.logdemo.iface.LogEx;

/**
 * Sample implementation of a component which produces log messages every few seconds.
 * 
 * This class uses its own thread, therefore it implements the {@link SOFALifecycle}
 * interface. It also supports dynamic updates and therefore it implements 
 * the {@link SOFAUpdatable} interface. To use the required {@link LogEx} interface,
 * it implements the {@link SOFAClient}.
 * 
 * @author Jaroslav Keznikl
 */
public class Tester implements SOFALifecycle, Runnable, SOFAClient, SOFAUpdatable {
	/** 
	 * Serial number of the log message.
	 * Represents the component's state. 
	 */	
	int serial = 0;
	/**
	 * Indicates that the inner thread should stop.
	 */
	boolean end = false;
	
	/**
	 * The required logger interface.
	 */
	LogEx logger = null;

	/**
	 * {@link SOFALifecycle#start()}
	 * Starts the inner execution thread.
	 */
	public void start() {
		Thread t = new Thread(this);
		t.start();
	}

	/**
	 * Ignored
	 */
	public void stopping() {
	}

	/**
	 * {@link SOFALifecycle#start()}
	 * Stops the inner execution thread.
	 */
	public void stop() {
		end = true;
	}

	/**
	 * The inner thread run method.
	 */
	public void run() {
		while (!end) {
			int a;
			try {
				a = logger.log(0, "Hello world from Java on level 0 (" + serial
						+ ")");
				a += logger.log(1, "Hello world from Java on level 1 ("
						+ serial + ")");
				a += logger.log(2, "Hello world from Java on level 2 ("
						+ serial + ")");
				serial++;
			} catch (LogDisabledException e) {
				e.printStackTrace();
				return;
			}
			System.out.println("Total: " + a);
			try {
				Thread.sleep(5000);
			} catch (Exception e) {
			}
		}
	}


	/**
	 * {@link SOFAClient#setRequired(String, Object)}
	 * 
	 * Sets the required 'logEx' interface reference.
	 */
	public void setRequired(String name, Object iface) {
		if (name.equals("logEx")) {
			if (iface instanceof LogEx) {
				logger = (LogEx) iface;
			}
		}
	}

	/**
	 * Loads the serial message number.
	 * 
	 * {@link SOFAUpdatable#loadState(Map)} 
	 */
	public void loadState(Map<String, Object> state) throws Throwable {
		serial = ((Double) state.get("serial")).intValue();
	}

	/**
	 * Saves the serial message number.
	 * 
	 * {@link SOFAUpdatable#loadState(Map)} 
	 */
	public void saveState(Map<String, Object> state) throws Throwable {
		state.put("serial", serial);
	}
}
