/* $Id$
 *
 * SOFA 2.0 component system
 * Copyright (C) 2006-2010  DDDS, Charles University in Prague
 *                          http://d3s.mff.cuni.cz/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA
 *
 */


/******************************************************************************
 * This file contains a sample implementation of the
 * org.ow2.dsrg.sofa.examples.script.logdemo.arch.Tester architecture
 * which produces log messages every few seconds.
 * 
 * This implementation uses its own thread, therefore it implements the 
 * {@link SOFALifecycle} interface. It also supports dynamic updates and 
 * therefore it implements the {@link SOFAUpdatable} interface. 
 * 
 * @author Jaroslav Keznikl
 *****************************************************************************/

/** True if the inner thread should stop. */
var end = false;

/** The inner execution thread */
var t;
/** The runnable implementation reference */
var v;

/** 
 * Serial number of the log message.
 * Represents the component's state. 
 */
var serial = 0;

/*
 * A dummy object for class loading test.
 * Instantiates a class form the dependent code bundle.
 */
var loaderTester = new org.ow2.dsrg.sofa.examples.script.logdemo.cbundle.Common.ClassLoaderTester();

// test the class loader
println('Testing class loader, tester message: ' + loaderTester.message());

/**
 * The main function of the execution thread.
 * Sends log messages every 5 seconds on 3 different log levels.
 * After each cycle increments the message counter.
 */
function main () {	
	var ret = 0;							
	try {
		// send log messages.
        ret = logEx.log(0, "Hello world from script on level 0 (" + serial + ")");
        ret += logEx.log(1, "Hello world from script on level 1 (" + serial + ")");
        ret += logEx.log(2, "Hello world from script on level 2 (" + serial + ")");
        
        // print the return value
        println("Printed " + ret + " characters");    
        
        // increment the message number	        
        serial++;
        
        // wait a few seconds
        java.lang.Thread.sleep(5000);        
        
	} catch (e) {
      	 println('While failed. ' + e.message + '\n' + e.stack);
      	 end = true;      	 
	}
}

/**
 * Starts the inner execution thread.
 * {@link SOFALifecycle#start()}
 */
function start() {
	println('Starting script-logdemo');
	
	// create an anonymous Runnable implementation instance
	v = new java.lang.Runnable() {
		run: function() {
			while (!end) {
				main();
			}
		}
	}
	
	try {
		// create and start the inner thread
		t = new java.lang.Thread(v);
    	t.start();
	} catch (e) {
		println('Run failed. ' + e.message);		
	}
}

/**
 * Stops the inner execution thread.
 * {@link SOFALifecycle#stop()}
 */
function stop() {
	end = true;
}

/**
 * Loads the serial message number.
 * {@link SOFAUpdatable#loadState(Map)} 
 */
function loadState(state) {
	serial = state.get("serial");
}

/**
 * Saves the serial message number.
 * {@link SOFAUpdatable#loadState(Map)} 
 */
function saveState(state) {
	state.put("serial", serial);
}
